import { calcActionItemAuthState, deepCopy, GridControlProps, GridControlState, IActionParam, IParam, MainControl } from '@ibiz-core';

/**
 * @description 表格部件
 * @export
 * @class GridControl
 * @extends {MainControl}
 */
export class GridControl extends MainControl {
  
  /**
   * @description 部件状态
   * @type {GridControlState}
   * @memberof GridControl
   */
  public declare controlState: GridControlState;

  /**
   * @description
   * @param {GridControlProps} props
   * @memberof GridControl
   */
  public setState(props: GridControlProps) {
    super.setState(props);
    this.controlState.multiple = toRef(props, 'multiple') as any;
    this.controlState.rowEditState = toRef(props, 'rowEditState') as any;
    this.controlState.rowActiveMode = toRef(props, 'rowActiveMode') as any;
    this.controlState.selectedData = toRef(props, 'selectedData') as any;
    this.controlState.selectFirstDefault = toRef(props, 'selectFirstDefault') as any;
  }

  /**
   * @description 表格数据改变
   * @param {number} rowIndex 行索引
   * @param {string} name 表格列属性名称
   * @param {*} value 表格列属性值
   * @memberof GridControl
   */
  public gridDataChange(rowIndex: number, name: string, value: any){
    this.controlState.items[rowIndex][name] = value;
  }

  /**
   * @description 获取界面行为权限状态
   * @param {IParam} rowData 行数据
   * @return {*} 
   * @memberof GridControl
   */
  public getActionAuthState(rowData: IParam) {
    const { UIService, actionModel } = this.controlState;
    let tempActionModel: any = deepCopy(actionModel);
    calcActionItemAuthState(rowData, tempActionModel, UIService);
    return tempActionModel;
  }

  /**
   * @description 获取指定列类型模型
   * @param {IParam[]} columns 表格列集合
   * @param {string} columnType 表格列类型
   * @return {*}  {*}
   * @memberof GridControl
   */
  public getGridColumn(columns: IParam[], columnType: string): any {
    columns.forEach((column: IParam) => {
      if (Object.is(column.columnType, columnType)) {
        return column;
      }
      if (column.children?.length > 0) {
        return this.getGridColumn(column.children, columnType);
      }
    })
  }

  /**
   * @description 自动分组
   * @memberof GridControl
   */
  public autoGroup() {
    const { groupField } = this.controlState;
    const dataRef = toRef(this.controlState, 'items');
    let autoGroup: string[] = [];
    dataRef.value.forEach((item: IParam) => {
      if (item.hasOwnProperty(groupField)) {
        autoGroup.push(item[groupField]);
      }
    })
    autoGroup = [...new Set(autoGroup)];
    if (autoGroup.length > 0) {
      const columnsModelRef = toRef(this.controlState, 'columnsModel');
      const groupColumn = this.getGridColumn(columnsModelRef.value, 'GROUP');
      const gridData: IParam[] = [];
      autoGroup.forEach((group: string) => {
        const children: IParam[] = [];
        dataRef.value.forEach((item: IParam) => {
          if (Object.is(group, item[groupField])) {
            children.push(item);
          }
        })
        gridData.push({
          [groupColumn.dataIndex]: group,
          children: deepCopy(children),
        })
      });
      dataRef.value = gridData;
    }
  }

  /**
   * @description 代码表分组
   * @memberof GridControl
   */
  public async codeListGroup() {
    const { groupField } = this.controlState;
    const dataRef = toRef(this.controlState, 'items');
    // TODO 代码表数据
    let codeListGroup: IParam[] = [];
    if (codeListGroup.length > 0) {
      const columnsModelRef = toRef(this.controlState, 'columnsModel');
      const groupColumn = this.getGridColumn(columnsModelRef.value, 'GROUP');
      const gridData: IParam[] = [];
      const otherGroup: IParam[] = [];
      codeListGroup.forEach((group: IParam) => {
        const children: IParam[] = [];
        dataRef.value.forEach((item: IParam) => {
          if (Object.is(group.value, item[groupField])) {
            children.push(item);
          }
        });
        gridData.push({
          [groupColumn.dataIndex]: group.text,
          children: deepCopy(children),
        })
      });
      dataRef.value.forEach((item: IParam) => {
        const index: number = codeListGroup.findIndex((_item: IParam) => Object.is(item[groupField], _item.value));
        if (index < 0) {
          otherGroup.push(item);
        }
      });
      if (otherGroup.length > 0) {
        gridData.push({
          [groupColumn.dataIndex]: "其它",
          children: deepCopy(otherGroup),
        })
      }
      dataRef.value = gridData;
    }
  }

  /**
   * @description 计算表格行为权限
   * @memberof GridControl
   */
  public calcGridAuthState() {
    const dataRef = toRef(this.controlState, 'items');
    const columnsModelRef = toRef(this.controlState, 'columnsModel');
    let columnModel = this.getGridColumn(columnsModelRef.value, 'UAGRIDCOLUMN');
    if (columnModel) {
      dataRef.value.forEach((item: IParam) => {
        Object.assign(item, {
          [columnModel.dataIndex]: this.getActionAuthState(item),
        })
      })
    }
  }

  /**
   * @description 处理表格分组
   * @memberof GridControl
   */
  public handleGridGroup() {
    const { groupMode, groupField } = this.controlState;
    if (groupField) {
      if (Object.is(groupMode, "AUTO")) {
        this.autoGroup();
      } else if (Object.is(groupMode, "CODELIST")) {
        this.codeListGroup();
      }
    }
  }

  /**
   * @description 远程聚合
   * @memberof GridControl
   */
  public async remoteAgg(): Promise<IParam[]> {
    const { aggEntity, aggDataSet } = this.controlState;
    const aggData: IParam[] = [];
    // TODO 调用服务获取聚合数据
    return aggData;
  }

  /**
   * @description 获取聚合值
   * @param {IParam[]} aggData 聚合数据
   * @param {string} column 列模型
   * @memberof GridControl
   */
  public getAggValue(aggData: IParam[], column: IParam): string {
    switch (column.aggMode) {
      case 'NONE':
        return '';
      case 'SUM':
        let sum: number = 0;
        aggData.forEach((item: IParam) => {
          sum += Number(item[column.dataIndex] ? item[column.dataIndex] : 0);
        })
        return 'SUM:' + sum;
      case 'AVG':
        let sum2: number = 0;
        aggData.forEach((item: IParam) => {
          sum2 += Number(item[column.dataIndex] ? item[column.dataIndex] : 0);
        })
        return 'AVG:' + Number(sum2 / (aggData.length)).toFixed(3);
      case 'MAX':
        let max: number = 0;
        aggData.forEach((item: IParam) => {
          const value: number = Number(item[column.dataIndex] ? item[column.dataIndex] : 0);
          if (value > max) {
            max = value;
          }
        })
        return 'MAX:' + max;
      case 'MIN':
        let min: number = 0;
        aggData.forEach((item: IParam) => {
          const value: number = Number(item[column.dataIndex] ? item[column.dataIndex] : 0);
          if (min > value) {
            min = value;
          }
        })
        return 'MIN:' + min;
      default:
        return '';
    }
  }

  /**
   * @description 处理数据聚合
   * @memberof GridControl
   */
  public async handleDataAgg() {
    const { aggMode } = this.controlState;
    if (!Object.is(aggMode, 'NONE')) {
      const dataAggRef = toRef(this.controlState, 'dataAgg');
      let aggData: IParam[];
      if (Object.is(aggMode, 'PAGE')) {
        const dataRef = toRef(this.controlState, 'items');
        aggData = [...dataRef.value];
      } else if (Object.is(aggMode, 'ALL')) {
        aggData = await this.remoteAgg();
      }
      const columnsModelRef = toRef(this.controlState, 'columnsModel');
      const columnsModel: IParam[] = [];
      columnsModelRef.value.forEach((column: IParam) => {
        if (!Object.is(column.columnType, 'GROUPGRIDCOLUMN')) {
          columnsModel.push(column);
        }
      });
      const dataAgg: string[] = [];
      columnsModel.forEach((column: IParam) => {
        dataAgg.push(this.getAggValue(aggData, column))
      })
      dataAggRef.value = [...dataAgg];
    }
  }

  /**
   * @description 使用加载功能模块
   * @param {GridControlProps} props 传入的props
   * @return {*} 
   * @memberof GridControl
   */
  public useLoad(props: GridControlProps){
    const { viewSubject, controlName } = this.controlState;
    const load = async (opt: any = {})=>{
      try {
        const loadAction = this.controlState.controlAction.loadAction;
        const { 
          controlService, context, viewParams, showBusyIndicator, noSort, minorSortDir, minorSortPSDEF,
          enablePagingBar, pageSize
        } = this.controlState;
        const dataRef = toRef(this.controlState, 'items');
        const paginationRef = toRef(this.controlState, 'pagination');
        if(!loadAction){
          return;
        }
        let _context = deepCopy(context);
        let _viewParams = deepCopy(viewParams);
        if (noSort && minorSortDir && minorSortPSDEF) {
          Object.assign(_viewParams, { sort: `${minorSortPSDEF},${minorSortDir}`});
        }
        if (enablePagingBar) {
          if (opt.current && opt.pageSize) {
            Object.assign(_viewParams, { page: opt.current - 1, size: opt.pageSize});
          } else {
            Object.assign(_viewParams, { page: 0, size: pageSize});
          }
        }
        const response = await controlService.get(loadAction, _context, {viewParams: _viewParams}, showBusyIndicator );
        if (!response.status || response.status !== 200) {
          return
        }
        dataRef.value = response.data;
        if (enablePagingBar) {
          paginationRef.value['total'] = response.total;
        }
        this.calcGridAuthState();
        this.handleGridGroup();
        this.handleDataAgg();
      } catch (error) {
        // todo 错误异常处理
      }
    }

    // 订阅viewSubject,监听load行为
    if(viewSubject){
      let subscription = viewSubject.subscribe(({ tag, action, data }: IActionParam)=>{
        if(Object.is(controlName, tag) && Object.is("load", action) ){
          load(data)
        }
      })
      // 部件卸载时退订viewSubject
      onUnmounted(()=>{
        subscription.unsubscribe();
      })
    }

    return {
      load: load
    }
  }

  /**
   * @description 处理编辑器事件
   * @param {IActionParam} actionParam 行为参数
   * @memberof GridControl
   */
  public handleEditorEvent(rowIndex: number, actionParam: IActionParam) {
    const { tag, action, data } = actionParam;
    switch (action) {
      case "valueChange":
        this.gridDataChange(rowIndex, tag, data);
        break;
      default:
        break;
    }
  }

  /**
   * @description 安装部件所有功能模块的方法
   * @param {GridControlProps} props 传入的Props
   * @param {Function} [emit]
   * @return {*} 
   * @memberof GridControl [emit] 事件
   */
  public moduleInstall(props: GridControlProps, emit?: Function) {
    const superParams = super.moduleInstall(props, emit);
    const { load } = this.useLoad(props);

    const handleEditorEvent = this.handleEditorEvent.bind(this);
    this.handleEditorEvent = (rowIndex: number,actionParam: IActionParam) => {
      handleEditorEvent(rowIndex, actionParam)
    }

    return {
      ...superParams,
      state: this.controlState,
      load,
      handleEditorEvent: this.handleEditorEvent,
    };
  }
}
